"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.LwcModuleResolverPlugin = void 0;
/*
 * Copyright (c) 2022, salesforce.com, inc.
 * All rights reserved.
 * SPDX-License-Identifier: MIT
 * For full license text, see the LICENSE file in the repo root or https://opensource.org/licenses/MIT
 */
var path_1 = require("path");
var lwcResolver = require('@lukethacoder/lwc-module-resolver');
var EMPTY_STYLE = (0, path_1.resolve)(__dirname, 'mocks', 'empty-style.js');
function isImplicitHTMLImport(importee, importer) {
    return ((0, path_1.extname)(importer) === '.js' &&
        (0, path_1.extname)(importee) === '.html' &&
        (0, path_1.dirname)(importer) === (0, path_1.dirname)(importee) &&
        (0, path_1.basename)(importer, '.js') === (0, path_1.basename)(importee, '.html'));
}
function isImplicitCssImport(importee, importer) {
    return ((0, path_1.extname)(importee) === '.css' &&
        (0, path_1.extname)(importer) === '.html' &&
        ((0, path_1.basename)(importee, '.css') === (0, path_1.basename)(importer, '.html') ||
            (0, path_1.basename)(importee, '.scoped.css') === (0, path_1.basename)(importer, '.html')));
}
/**
 * Webpack plugin to resolve LWC modules.
 */
var LwcModuleResolverPlugin = /** @class */ (function () {
    function LwcModuleResolverPlugin(modules) {
        this.modules = modules;
    }
    LwcModuleResolverPlugin.prototype.apply = function (resolver) {
        var _this = this;
        this.fs = resolver.fileSystem;
        resolver.hooks.module.tapAsync('LWC module', function (req, ctx, cb) { return _this.resolveModule(req, ctx, cb); });
        resolver.hooks.file.tapAsync('LWC CSS', function (req, ctx, cb) {
            return _this.resolveFile(req, ctx, cb);
        });
    };
    LwcModuleResolverPlugin.prototype.resolveModule = function (req, ctx, cb) {
        var request = req.request, 
        // eslint-disable-next-line prefer-const
        query = req.query, issuer = req.context.issuer;
        try {
            if (!issuer) {
                issuer = process.cwd();
            }
            request = request.replace('./', '');
            var mod = void 0;
            if (this.modules && this.modules.length) {
                mod = lwcResolver.resolveModule(request, issuer, {
                    modules: this.modules,
                });
            }
            else {
                mod = lwcResolver.resolveModule(request, issuer);
            }
            return cb(undefined, {
                path: mod.entry,
                query: query,
                file: true,
                resolved: true,
            });
        }
        catch (e) {
            // LWC Module Resolver will throw errors for any non lwc modules
            cb();
        }
    };
    LwcModuleResolverPlugin.prototype.resolveFile = function (req, ctx, cb) {
        var resourcePath = req.path, query = req.query;
        var extFilename = (0, path_1.extname)(resourcePath);
        if (extFilename !== '.css' && extFilename !== '.html') {
            return cb();
        }
        this.fs.stat(resourcePath, function (err) {
            if (err !== null && err.code === 'ENOENT') {
                var isCSS = extFilename === '.css';
                if (isCSS && !isImplicitCssImport(resourcePath, req.context.issuer)) {
                    // warning for a missing non-implicit CSS import
                    console.warn("The imported CSS file ".concat(resourcePath, " does not exist: Importing it as undefined."));
                }
                if (isCSS || isImplicitHTMLImport(resourcePath, req.context.issuer)) {
                    return cb(null, {
                        path: EMPTY_STYLE,
                        query: query,
                        file: true,
                        resolved: false,
                    });
                }
            }
            return cb();
        });
    };
    return LwcModuleResolverPlugin;
}());
exports.LwcModuleResolverPlugin = LwcModuleResolverPlugin;
